//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "BackupDlgs.h"
#include "MsgBoxDlgs.h"
#include "Fileman.h"
#include "Global.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TBackupDlg *BackupDlg;
//---------------------------------------------------------------------------
bool __fastcall BackupQuickSilverData(TForm *Owner, bool SecureMode)
{
   BackupDlg = new TBackupDlg(Owner,SecureMode);
   BackupDlg->ShowModal();
   bool ok = BackupDlg->ModalResult==mrOk;
   delete BackupDlg;
   return ok;
}
//---------------------------------------------------------------------------
// TBackupThread
//---------------------------------------------------------------------------
__fastcall TBackupThread::TBackupThread(bool CreateSuspended)
	: TThread(CreateSuspended)
{
   finished = false;
   failed = false;
}
//---------------------------------------------------------------------------
void __fastcall TBackupThread::Execute()
{
   if (!CopyFile(ExistingFile.c_str(),CopyToFile.c_str(),false))
   {
      ErrorMsg = GetLastErrorStr();
      failed = true;
   }
   finished = true;
}
//---------------------------------------------------------------------------
// TBackupDlg
//---------------------------------------------------------------------------
__fastcall TBackupDlg::TBackupDlg(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TBackupDlg::TBackupDlg(TComponent* Owner, bool SecureMode)
	: TForm(Owner)
{
   securemode = SecureMode; CenterDialog(this,Owner);
   if (!FileExists(AppPgp)) PgpDB->Checked = PgpDB->Enabled = false;

   AnsiString db,bakinfo = GetAppIni("BackupPathName",ActiveDB()+"|"+ActiveDB()+".bak");
   db = GetToken(0,bakinfo,"|"); BackupFile->Text = GetToken(1,bakinfo,"|");

   if (db==AppDat.LowerCase() || !PgpDB->Enabled) DbcDB->Checked = true;
   else PgpDB->Checked = true;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TBackupDlg::ActiveDB()
{
   return s(securemode?AppPgp:AppDat).LowerCase();
}
//---------------------------------------------------------------------------
AnsiString __fastcall TBackupDlg::ChosenDB()
{
   if (PgpDB->Checked) return AppPgp.LowerCase();
   else return AppDat.LowerCase();
}
//---------------------------------------------------------------------------
void __fastcall TBackupDlg::OkBtnClick(TObject *Sender)
{
   if (ConfirmBox(this,"Copy database now?",MB_OKCANCEL)==IDCANCEL) return;

   AnsiString cap=Caption,copyname=BackupFile->Text.LowerCase();
   if (copyname.LowerCase()==ActiveDB())
   {
      MsgBoxHwnd(Handle,"Cannot copy file onto itself!","Filename Error",MB_OK);
      return;
   }
   Caption = "Saving...";
   Timer1->Enabled = true;
   CancelBtn->Enabled = OkBtn->Enabled = BrowseBtn->Enabled = BackupFile->Enabled =
      DbcDB->Enabled = PgpDB->Enabled = false;

   TDateTime now; //int c=0;
   TBackupThread *BackupThrd = new TBackupThread(true);
   BackupThrd->ExistingFile = ChosenDB();
   BackupThrd->CopyToFile = copyname;
   BackupThrd->Resume();

   int curr,ticks=GetTickCount();
   while (((curr=GetTickCount())>ticks && curr<ticks+1000) || !BackupThrd->finished)
      AppProcessMessages();

   /*while (c<60 || !BackupThrd->finished)
   {
      now = Time();
      while (now==Time())
         AppProcessMessages();
      c++;
   }*/

   if (BackupThrd->failed)
   {
      Sleep(200);
      MsgBoxHwnd(Handle,
         s("System Error: "+BackupThrd->ErrorMsg)+
         s("File: "+BackupFile->Text),
         "Copy Failed",
         MB_OK
      );
      delete BackupThrd;
      CancelBtn->Enabled = OkBtn->Enabled = BrowseBtn->Enabled = BackupFile->Enabled =
         DbcDB->Enabled = false;
      PgpDB->Enabled = FileExists(AppPgp);
      Timer1->Enabled = false;
      Label1->Caption = cap;
      return;
   }
   delete BackupThrd;

   Timer1->Enabled = false;
   Caption = cap;
   BingBox(this,"Backup Complete!              ","Success",MB_OK);
   Label1->Caption = "Ok! That was very good!";
   CancelBtn->Caption = "Close";
   CancelBtn->Enabled = OkBtn->Enabled = BrowseBtn->Enabled = BackupFile->Enabled =
      DbcDB->Enabled = true;
   PgpDB->Enabled = FileExists(AppPgp);
   AppProcessMessages();
   Sleep(200);
}
//---------------------------------------------------------------------------
void __fastcall TBackupDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (Timer1->Enabled)
   {
      CanClose = false;
      return;
   }
   if (ModalResult==mrOk) PutAppIni("BackupPathName",ChosenDB()+"|"+BackupFile->Text);
}
//---------------------------------------------------------------------------
void __fastcall TBackupDlg::Timer1Timer(TObject *Sender)
{
   if (Caption.IsEmpty()) Caption = "Saving...";
   else Caption = "";
   AppProcessMessages();
}
//---------------------------------------------------------------------------
void __fastcall TBackupDlg::CancelBtnClick(TObject *Sender)
{
   if (CancelBtn->Caption=="Close") ModalResult = mrOk;
   else ModalResult = mrCancel;
}
//---------------------------------------------------------------------------
void __fastcall TBackupDlg::BrowseBtnClick(TObject *Sender)
{
   SaveDialog->FileName = BackupFile->Text;
   SaveDialog->Filter = "";
   if (SaveDialog->Execute()) BackupFile->Text = SaveDialog->FileName;
}
//---------------------------------------------------------------------------
